import {Component, Input, OnInit} from '@angular/core';
import {MatTableDataSource} from "@angular/material/table";
import {DatePipe} from "@angular/common";
import {MatDialog} from "@angular/material/dialog";
import {HttpClient} from "@angular/common/http";
import {ToastrService} from "ngx-toastr";
import {JoyrideService} from "ngx-joyride";
import {Observable} from "rxjs";
import {DeleteModalComponent} from "../../../shared/delete_modal/delete-modal/delete-modal.component";
import {ExportImageComponent} from "../../images/export-image/export-image.component";
import {BackupImageComponent} from "../backup-image/backup-image.component";
import {ServerInfoDialogComponent} from "../../ogdhcp/server-info-dialog/server-info-dialog.component";

@Component({
  selector: 'app-repository-images',
  templateUrl: './repository-images.component.html',
  styleUrl: './repository-images.component.css'
})
export class RepositoryImagesComponent implements OnInit {
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  dataSource = new MatTableDataSource<any>();
  length: number = 0;
  itemsPerPage: number = 10;
  page: number = 0;
  loading: boolean = false;
  filters: { [key: string]: string } = {};
  alertMessage: string | null = null;
  repository: any = {};
  datePipe: DatePipe = new DatePipe('es-ES');
  columns = [
    {
      columnDef: 'id',
      header: 'Id',
      cell: (image: any) => `${image.id}`
    },
    {
      columnDef: 'image.image.name',
      header: 'Nombre de imagen',
      cell: (image: any) => `${image.image.name}`
    },
    {
      columnDef: 'remotePc',
      header: 'Remote Pc',
      cell: (image: any) => `${image.remotePc}`
    },
    {
      columnDef: 'status',
      header: 'Estado',
      cell: (image: any) => `${image.status}`
    },
    {
      columnDef: 'imageFullsum',
      header: 'Fullsum',
      cell: (image: any) => `${image.imageFullsum}`
    },
    {
      columnDef: 'createdAt',
      header: 'Fecha de creación',
      cell: (image: any) => `${this.datePipe.transform(image.createdAt, 'dd/MM/yyyy hh:mm:ss')}`
    }
  ];
  displayedColumns = [...this.columns.map(column => column.columnDef), 'actions'];

  private apiUrl = `${this.baseUrl}/image-image-repositories`;
  @Input() repositoryUuid: any
  private repositoryId: any;

  constructor(
    public dialog: MatDialog,
    private http: HttpClient,
    private toastService: ToastrService,
    private joyrideService: JoyrideService,
  ) {}

  ngOnInit(): void {
    if (this.repositoryUuid) {
      this.loadRepository()
    } else {
      this.search();
    }
  }

  loadRepository(): void {
    this.http.get<any>(`${this.baseUrl}/image-repositories/${this.repositoryUuid}`, {}).subscribe(
      data => {
        this.repositoryId = data.id;
        this.repository = data
        this.search();
      },
      error => {
        console.error('Error fetching image repositories', error);
      }
    )
  }

  getStatusLabel(status: string): string {
    switch (status) {
      case 'pending':
        return 'Pendiente';
      case 'in-progress':
        return 'En progreso';
      case 'aux-files-pending':
        return 'Archivos auxiliares pendientes';
      case 'success':
        return 'Creado con éxito';
      case 'trash':
        return 'Papelera temporal';
      case 'failed':
        return 'Fallido';
      case 'transferring':
        return 'Transfiriendo';
      default:
        return 'Estado desconocido';
    }
  }

  search(): void {
    this.loading = true;
    this.http.get<any>(`${this.apiUrl}?page=${this.page +1 }&itemsPerPage=${this.itemsPerPage}&repository.id=${this.repositoryId}`, { params: this.filters }).subscribe(
      data => {
        this.dataSource.data = data['hydra:member'];
        this.length = data['hydra:totalItems'];
        this.loading = false;
      },
      error => {
        console.error('Error fetching images', error);
        this.loading = false;
      }
    );
  }

  onPageChange(event: any): void {
    this.page = event.pageIndex;
    this.itemsPerPage = event.pageSize;
    this.length = event.length;
    this.search();
  }

  loadImageAlert(image: any): Observable<any> {
    return this.http.get<any>(`${this.apiUrl}/server/${image.uuid}/get`, {});
  }

  showImageInfo(event: MouseEvent, image:any) {
    event.stopPropagation();
    this.loading = true;
    this.loadImageAlert(image).subscribe(
      response => {
        this.alertMessage = response;

        this.dialog.open(ServerInfoDialogComponent, {
          width: '800px',
          data: {
            message: this.alertMessage
          }
        });
        this.loading = false;
      },
      error => {
        this.toastService.error(error.error['hydra:description']);
        this.loading = false;
      }
    );
  }

  toggleAction(image: any, action:string): void {
    switch (action) {
      case 'get-aux':
        this.http.post(`${this.baseUrl}/image-image-repositories/server/${image.uuid}/create-aux-files`, {}).subscribe({
          next: () => {
            this.toastService.success('Petición de creación de archivos auxiliares enviada');
            this.search()
          },
          error: (error) => {
            this.toastService.error(error.error['hydra:description']);
          }
        });
        break;
      case 'delete-trash':
        if (!image.imageFullsum) {
          const dialogRef = this.dialog.open(DeleteModalComponent, {
            width: '400px',
            data: { name: image.name },
          });

          dialogRef.afterClosed().subscribe((result) => {
            this.http.delete(`${this.baseUrl}${image['@id']}`).subscribe({
              next: () => {
                this.toastService.success('Image deleted successfully');
                this.search()
              },
              error: (error) => {
                this.toastService.error('Error deleting image');
              }
            });
          });
        } else {
          this.http.post(`${this.baseUrl}/image-image-repositories/server/${image.uuid}/delete-trash`,
            { repository: `/image-repositories/${this.repositoryUuid}` })
            .subscribe({
              next: () => {
                this.toastService.success('Petición de eliminación de la papelera temporal enviada');
                this.search()
              },
              error: (error) => {
                this.toastService.error(error.error['hydra:description']);
              }
            });
        }

        break;
      case 'delete-permanent':
        this.dialog.open(DeleteModalComponent, {
          width: '300px',
          data: { name: image.name },
        }).afterClosed().subscribe((result) => {
          if (result) {
            this.http.post(`${this.baseUrl}/image-image-repositories/server/${image.uuid}/delete-permanent`, {}).subscribe({
              next: () => {
                this.toastService.success('Petición de eliminación de la papelera temporal enviada');
                this.search()
              },
              error: (error) => {
                this.toastService.error(error.error['hydra:description']);
              }
            });
          }
        });
        break;
      case 'recover':
        this.http.post(`${this.baseUrl}/image-image-repositories/server/${image.uuid}/recover`, {}).subscribe({
          next: () => {
            this.toastService.success('Petición de recuperación de la imagen enviada');
            this.search()
          },
          error: (error) => {
            this.toastService.error(error.error['hydra:description']);
          }
        });
        break;
      case 'status':
        this.http.post(`${this.baseUrl}/image-image-repositories/server/${image.uuid}/status`, {}).subscribe({
          next: () => {
            this.toastService.success('Integridad de la imagen comprobada con éxito');
            this.search()
          },
          error: (error) => {
            this.toastService.error(error.error['hydra:description']);
          }
        });
        break;
      case 'transfer':
        this.http.get(`${this.baseUrl}${image.image['@id']}`).subscribe({
          next: (response) => {
            this.dialog.open(ExportImageComponent, {
              width: '600px',
              data: {
                image: response,
                imageImageRepository: image
              }
            });
          },
          error: (error) => {
            this.toastService.error(error.error['hydra:description']);
          }
        });
        break;
      case 'backup':
        this.http.get(`${this.baseUrl}${image.image['@id']}`).subscribe({
          next: (response) => {
            this.dialog.open(BackupImageComponent, {
              width: '600px',
              data: {
                image: response,
                imageImageRepository: image
              }
            });
          },
          error: (error) => {
            this.toastService.error(error.error['hydra:description']);
          }
        });
        break;
      default:
        console.error('Acción no soportada:', action);
        break;
    }
  }
  iniciarTour(): void {
    this.joyrideService.startTour({
      steps: [
        'imagesTitleStep',
        'addImageButton',
        'searchImagesField',
        'imagesTable',
        'actionsHeader',
        'editImageButton',
        'deleteImageButton',
        'imagesPagination'
      ],
      showPrevButton: true,
      themeColor: '#3f51b5'
    });
  }
}
